/***************************************************************************
                          ImageBox.h  -  description
                             -------------------
    begin                : ven mag 7 2004
    copyright            : (C) 2004 by Daniele Lavatelli
    email                : ldaniele@interfree.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef IMAGE_BOX_H

 #define IMAGE_BOX_H

 #include <qimage.h>
 #include <qpopupmenu.h>
 #include <qevent.h>
 #include <qpixmap.h>
 #include <qlabel.h>
 #include <qframe.h>
 #include <pthread.h>

 #include "Net.h"

 enum ErrorType  {ERR_WARNING,ERR_CRITICAL,ERR_NOERRORS};
 
 class ImageBox : public QWidget
 {

  Q_OBJECT
  
  public:
  ImageBox(QWidget *,const char *,int,int);
  ~ImageBox();
  void Select();
  void Deselect();
  void AutoConnect(const char *,const char *);
  void AdjustSize();
  char *GetServerName();
  static void* StartThread(void*);

  signals:
   void StatusMsg(ErrorType,char *);  //emitted in Selected status only
   void Clicked(int,int);
   void FrameRateMsg(float);
  
  private:
  QPopupMenu      *mpMenu;
  QImage          mImage;
  QImage          temp_frame;
  QWidget         *mpBox;
  JpegFrameType   mJpegFrame;
  Connection      mConnection;
  void            Loop();
  void            SetErrorStatus(ErrorType,char *);
  pthread_t       mThId;
  pthread_mutex_t mBusy;
  bool            mDrawActive;
  int             stretch_id,fit_id,normal_id;
  int             connect_id,reconnect_id,disconnect_id,inspect_id;
  ErrorType       mErrorStatus;
  char            mStatusMessage[255];
  bool            mSelected;
  int             mX,mY;
  char            mServerName[255];
  char            mServerPort[10];
  
  private slots:
  void ShowStretch();
  void ShowFit();
  void ShowNormal();
  void Connect();
  void Reconnect();
  void Disconnect();
  void Inspect();

  protected:
  void mousePressEvent( QMouseEvent *e );
  void resizeEvent( QResizeEvent *e );
  void timerEvent( QTimerEvent *e );
  void paintEvent( QPaintEvent *e );
  
 };

 
#endif
