
/*
    YUV->RGB conversion.

    Copyright 2000-2003 by David Santinoli <david@santinoli.com>


    Possible improvements:
    - look-up tables
    - use SIMD instructions (MMX etc.)

 
    Reference: http://cedar.intel.com/cgi-bin/ids.dll/content/content.jsp?cntKey=Legacy::irtm_COLORCON_10809&cntType=IDS_EDITORIAL#6

    420p = "4:2:0 planar"
            YYYYYYYY      N lines
            UUUU          N/2 lines
            VVVV          N/2 lines

    G = 1.164*(Y-16) -0.813*(V-128) -0.391*(U-128)
    B = 1.164*(Y-16)                +2.018*(U-128)
    R = 1.164*(Y-16) +1.596*(V-128)

*/

#include "video.h"

#define clip(x) ( (x)<0 ? 0 : ( (x)>255 ? 255 : (x) ) )

static const char rcsid[] = "$Id: yuv2rgb.c,v 1.6 2003/09/20 22:04:32 santinod Exp $";

void yuv2rgb (unsigned char *out, unsigned char *in, int width, int height)
{
 unsigned char *u,*u1,*v,*v1;
 int Y=0,U=0,V=0,R,G,B,i,j;

 u=in+width*height;
 v=u+(width*height)/4;

 for(i=0;i<height;i++) {
    u1=u;
    v1=v;
    for(j=0;j<width;j++) {
       Y=(*in++)-16;
       if((j&1)==0) {
          U=(*u++)-128;
          V=(*v++)-128;
          }

       R=(76284*Y+104595*V)>>16;
       G=(76284*Y -25625*U-53281*V)>>16;
       B=(76284*Y+132252*U)>>16;

       *out++=clip(R);
       *out++=clip(G);
       *out++=clip(B);
       }

    if((i&1)==0) { u=u1; v=v1; }
    }
}
